require 'spec_helper'

RSpec.describe NsSettingsUi::Setting, type: :model do
  describe '.get' do
    let(:key) { 'matching.cluster.size' }

    context 'when setting exists' do
      let!(:setting) { described_class.create(key: key) }

      it 'finds setting by country and exact key' do
        expect(described_class.get(key)).to eq(setting)
      end

      it 'finds setting by country and fragmented key' do
        expect(described_class.get(*key.split('.'))).to eq(setting)
      end
    end

    context 'when setting does not exist' do
      it 'creates setting with empty value' do
        expect(described_class.get(key).value).to be_nil
      end

      it 'creates setting for the given key' do
        expect(described_class.get(key).key).to eq(key)
      end
    end
  end

  describe '#keys' do
    subject { described_class.create(key: key) }

    context 'when key is empty' do
      let(:key) { '' }

      its(:keys) { is_expected.to be_empty }
    end

    context 'when there is a simple key' do
      let(:key) { 'aa' }

      its(:keys) { is_expected.to eq([:aa]) }
    end

    context 'when key consists of multiple keys' do
      let(:key) { 'aa.bb' }

      its(:keys) { is_expected.to eq([:aa, :bb]) }
    end
  end

  describe '#formatted_value' do
    let(:key) { 'aa.bb' }
    subject { described_class.create(key: key, value: value, value_type: value_type).formatted_value }

    shared_examples 'string' do
      context 'when value is a string' do
        let(:value) { 'abcd' }
        it { is_expected.to eq('abcd') }
      end

      context 'when value is an integer with a string' do
        let(:value) { '4abcd' }
        it { is_expected.to eq('4abcd') }
      end

      context 'when value is an integer' do
        let(:value) { '10' }
        it { is_expected.to eq('10') }
      end
    end

    shared_examples 'integer' do
      context 'when value is a string' do
        let(:value) { 'abcd' }
        it { is_expected.to eq(0) }
      end

      context 'when value is an integer with a string' do
        let(:value) { '4abcd' }
        it { is_expected.to eq(4) }
      end

      context 'when value is an integer' do
        let(:value) { '10' }
        it { is_expected.to eq(10) }
      end
    end

    shared_examples 'boolean' do
      context 'when value is true' do
        let(:value) { 'true' }
        it { is_expected.to eq(true) }
      end

      context 'when value is false' do
        let(:value) { 'false' }
        it { is_expected.to eq(false) }
      end

      context 'when value is other than true or 1 or false' do
        let(:value) { 'other' }
        it { is_expected.to eq(false) }
      end
    end

    shared_examples 'nullable_type' do
      context 'when value is a nil' do
        let(:value) { nil }
        it { is_expected.to eq(nil) }
      end

      context 'when value is a empty string' do
        let(:value) { '' }
        it { is_expected.to eq(nil) }
      end
    end

    context 'when value_type is default' do
      let(:value_type) { nil }

      context 'when value is a nil' do
        let(:value) { nil }
        it { is_expected.to be_nil }
      end

      context 'when value is a string' do
        let(:value) { 'abcd' }
        it { is_expected.to eq('abcd') }
      end

      context 'when value is an integer with a string' do
        let(:value) { '4abcd' }
        it { is_expected.to eq('4abcd') }
      end

      context 'when value is an integer' do
        let(:value) { '10' }
        it { is_expected.to eq('10') }
      end

      context 'when value is a boolean' do
        let(:value) { 'true' }
        it { is_expected.to eq('true') }
      end
    end

    context 'when value_type is :string' do
      let(:value_type) { NsSettingsUi::Setting::TYPES[:string] }

      context 'when value is a nil' do
        let(:value) { nil }
        it { is_expected.to eq('') }
      end

      it_behaves_like 'string'
    end

    context 'when value_type is :nullable_string' do
      let(:value_type) { NsSettingsUi::Setting::TYPES[:nullable_string] }

      it_behaves_like 'string'
      it_behaves_like 'nullable_type'
    end

    context 'when value_type is :integer' do
      let(:value_type) { NsSettingsUi::Setting::TYPES[:integer] }

      context 'when value is a nil' do
        let(:value) { nil }
        it { is_expected.to eq(0) }
      end

      it_behaves_like 'integer'
    end

    context 'when value_type is :nullable_integer' do
      let(:value_type) { NsSettingsUi::Setting::TYPES[:nullable_integer] }

      it_behaves_like 'integer'
      it_behaves_like 'nullable_type'
    end

    context 'when value_type is :boolean' do
      let(:value_type) { NsSettingsUi::Setting::TYPES[:boolean] }

      context 'when value is a nil' do
        let(:value) { nil }
        it { is_expected.to eq(false) }
      end

      it_behaves_like 'boolean'
    end

    context 'when value_type is :nullable_boolean' do
      let(:value_type) { NsSettingsUi::Setting::TYPES[:nullable_boolean] }

      it_behaves_like 'boolean'
      it_behaves_like 'nullable_type'
    end

    context 'when value_type is :date' do
      let(:value_type) { NsSettingsUi::Setting::TYPES[:date] }

      context 'when value is a proper date' do
        let(:value) { '2017-05-08' }
        it { is_expected.to eq(Date.parse(value)) }
      end

      context 'when value is malformatted' do
        let(:value) { '2017-05-df' }
        it { expect { subject }.to raise_error(ArgumentError, 'invalid date') }
      end
    end
  end
end
